-- Procesador sin Memoria Externa ni Perifricos.
-- Copia de micro8_1, para meter el divisor de frecuencia y el buffer triestado
-- para el acceso al bus de datos.
-- 19 de Junio del 2001.
PACKAGE ram_constants IS
	constant ADDR_WIDTH: INTEGER := 4;
	constant DATA_WIDTH: INTEGER := 16;
END ram_constants;

LIBRARY ieee;
LIBRARY lpm;
LIBRARY work;

USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_signed.all;
USE lpm.lpm_components.all;
USE work.ram_constants.all;

-- Cambiar los puertos
ENTITY code_2 IS
PORT(clock,reset,ireq:IN std_logic;
	 AR:OUT std_logic_vector(15 downto 0);
	 tri_out:OUT std_logic_vector(15 downto 0);
	 we,ce:OUT std_logic;--las dems seales de MP estn siempre bajas 
	 qMem:IN std_logic_vector(15 downto 0));-- Hace de salida de memoria principal, M(AR)
END code_2;								

ARCHITECTURE x1 OF code_2 IS

COMPONENT div_frec
PORT (clock:IN STD_LOGIC;
	  clk_out:OUT STD_LOGIC);
END COMPONENT;

COMPONENT lc
PORT(reloj,reset:IN std_logic;
	 FF:IN std_logic_vector(15 downto 0);
	 ireq:IN std_logic;
	 codop,cnd:IN std_logic_vector(3 downto 0);
	 we,ce:OUT std_logic;--las dems seales estn siempre bajas 
	 b,c_1,c_e,d,e,f,g,h,i,ir0,ir1,ir2:OUT std_logic;
	 k,l,m,n,p,q,s_FF,t,u,v,x,y:OUT std_logic);
END COMPONENT;

COMPONENT sel1
PORT(c_1:IN std_logic;
	 sel1_A,sel1_B:IN std_logic_vector(15 DOWNTO 0);
	 sel1_out:OUT std_logic_vector(15 DOWNTO 0));
END COMPONENT;

COMPONENT sel2
PORT(u,v,x:IN std_logic;
	 IR_11_0:IN std_logic_vector(11 DOWNTO 0);
	 sel2_out:OUT std_logic_vector(3 DOWNTO 0));
END COMPONENT;

COMPONENT sel4
PORT(k,l:IN std_logic;
	 sel4_in:IN std_logic_vector(15 DOWNTO 0);
	 sel4_out:OUT std_logic_vector(15 DOWNTO 0));
END COMPONENT;

COMPONENT sel5
PORT(ir0,ir1:IN std_logic;
	 sel5_A:IN std_logic_vector(15 DOWNTO 0);
	 sel5_out:OUT std_logic_vector(15 DOWNTO 0));
END COMPONENT;

COMPONENT sel6
PORT(ir2:IN std_logic;
	 sel6_A:IN std_logic_vector(15 DOWNTO 0);
	 sel6_out:OUT std_logic_vector(15 DOWNTO 0));
END COMPONENT;

COMPONENT dec_bin
PORT(b,h:IN std_logic;
	 qReg,DR,PC,IR: IN STD_LOGIC_VECTOR(15 DOWNTO 0);
	 B2:OUT STD_LOGIC_VECTOR(15 DOWNTO 0));
END COMPONENT;

COMPONENT registro
PORT(bus_in:IN std_logic_vector(15 downto 0);
	 enable:IN std_logic;
	 bus_out:OUT std_logic_vector(15 downto 0));
END COMPONENT;

COMPONENT tri_est
PORT (tri_in:IN std_logic_vector(15 downto 0);
	  control:IN std_logic;
	  tri_out:OUT std_logic_vector(15 downto 0));
END COMPONENT;

COMPONENT reg_FF
PORT(Z,S,C,W:IN std_logic;
	 enable:IN std_logic;
	 FF:OUT std_logic_vector(15 downto 0));
END COMPONENT;

COMPONENT alu
GENERIC (n_bits:integer:=16);
PORT(m,n,p,q:IN std_logic;
	 Z,S,C,W:OUT std_logic;
	 BA,BB:IN std_logic_vector(n_bits-1 DOWNTO 0);
	 B1:OUT std_logic_vector(n_bits-1 DOWNTO 0));
END COMPONENT;

-- Seales de LC
SIGNAL codop,cnd:std_logic_vector(3 downto 0);
SIGNAL FF:std_logic_vector(15 downto 0);
SIGNAL reloj,b,c_1,c_e,d,e,f,g,h,i,ir0,ir1,ir2,k,l,m,n,p,q,s_FF,t,u,v,x,y:STD_LOGIC;

-- Seales del banco de registros
SIGNAL qReg,data:std_logic_vector(15 downto 0);
SIGNAL sel2_out:std_logic_vector(3 downto 0);
SIGNAL IR_11_0:std_logic_vector(11 downto 0);

-- Seales de los selectores
SIGNAL sel1_out,sel4_in,sel5_out,sel6_out:std_logic_vector(15 downto 0);

-- Seales del decodificador binario
SIGNAL DR,PC,IR,B2:std_logic_vector(15 downto 0);

-- Seales de la ALU
SIGNAL B1,BA,BB:std_logic_vector(15 downto 0);
SIGNAL Z,S,C,W:std_logic;

BEGIN

frec_2 : div_frec
		 PORT MAP(clock=>clock,clk_out=>reloj);

control: lc
		 PORT MAP (reloj=>reloj,reset=>reset,ireq=>ireq,codop=>IR(15 downto 12),cnd=>IR(11 downto 8),
				   FF=>FF,we=>we,ce=>ce,b=>b,c_1=>c_1,c_e=>c_e,d=>d,e=>e,f=>f,g=>g,h=>h,i=>i,ir0=>ir0,
				   ir1=>ir1,ir2=>ir2,k=>k,l=>l,m=>m,n=>n,p=>p,q=>q,s_FF=>s_FF,t=>t,u=>u,v=>v,
				   x=>x,y=>y);

bancoRF: LPM_RAM_DQ
		 GENERIC MAP (lpm_widthad=>ADDR_WIDTH,lpm_width=>DATA_WIDTH,
					  LPM_INDATA=>UNREGISTERED,LPM_ADDRESS_CONTROL=>UNREGISTERED,
      				  LPM_OUTDATA=>UNREGISTERED)
		 PORT MAP (data=>B1,address=>sel2_out,we=>y,q=>qReg);

select1: sel1
		 PORT MAP (c_1=>c_1,sel1_A=>B1,sel1_B=>qMem,sel1_out=>sel1_out);

select2: sel2
		 PORT MAP (u=>u,v=>v,x=>x,IR_11_0=>IR(11 downto 0),sel2_out=>sel2_out);

select4: sel4
		 PORT MAP (k=>k,l=>l,sel4_in=>sel4_in,sel4_out=>BA);

select5: sel5
		 PORT MAP (ir0=>ir0,ir1=>ir1,sel5_A=>B1,sel5_out=>sel5_out);

select6: sel6
		 PORT MAP (ir2=>ir2,sel6_A=>B1,sel6_out=>sel6_out);

deco_bh: dec_bin
		 PORT MAP (b=>b,h=>h,qReg=>qReg,DR=>DR,PC=>PC,IR=>IR,B2=>B2);

reg_DR : registro 
	     PORT MAP (bus_in=>sel1_out,enable=>d,bus_out=>DR);

triesta: tri_est
		 PORT MAP(tri_in=>DR,control=>t,tri_out=>tri_out);

reg_AR : registro 
	     PORT MAP (bus_in=>B1,enable=>f,bus_out=>AR);

reg_IR : registro 
	     PORT MAP (bus_in=>sel6_out,enable=>i,bus_out=>IR);

reg_PC : registro
	     PORT MAP (bus_in=>sel5_out,enable=>g,bus_out=>PC);

reg_RT : registro
	     PORT MAP (bus_in=>B2,enable=>e,bus_out=>BB);

reg_C1 : registro
	     PORT MAP (bus_in=>B2,enable=>c_e,bus_out=>sel4_in);

FF_reg : reg_FF
	     PORT MAP (Z=>Z,S=>S,C=>C,W=>W,enable=>s_FF,FF=>FF);

alu_0  : alu
		 PORT MAP(m=>m,n=>n,p=>p,q=>q,Z=>Z,S=>S,C=>C,W=>W,BA=>BA,BB=>BB,B1=>B1);
END x1;
-- 747 LCs (64 %) 
-- SPEED = 10

-- Para poder ver los registros PC, IR, DR, RT y C1 en la simulacin,
-- habra que poner el 'bus_out' del componente 'registro' en modo 'INOUT'.

